from geezlibs.autobot import *
from geezlibs._wrappers import *


from geezram import __version__ as grver
from Geez import cmds
__version__ = "0.0.1"
bot_version = "0.1.5"

tempe_orek = """
**Geez Pyro Userbot**
╼┅━━━━━━━━━━╍━━━━━━━━━━┅╾
**Userbot Version -** `{bot_version}`
**GeezRam Version -** `{grver}`
**Geezlibs Version -** `{__version__}`
**Ketik** `{cmds}geez` **untuk Mengecheck Bot**
╼┅━━━━━━━━━━╍━━━━━━━━━━┅╾
©️2023 Geez|RAM Projects
"""
tempe_bacem = (f"**Geez Pyro Assistant**\nis alive...")